<?php
require_once('../vendor/autoload.php');
require_once('../Constant.php');

use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;

$queue_name = 'hello_queue';

// 创建连接
$connection = new AMQPStreamConnection(
    Constant::HOST,
    Constant::PORT,
    Constant::USERNAME,
    Constant::PASSWORD,
    Constant::VHOST,
    false,
    Constant::LOGIN_METHOD);
// 建立通道
$channel = $connection->channel();

// 开启发布者确认
$channel->confirm_select();

// ack回调
$channel->set_ack_handler(
    function (AMQPMessage $message) {
        // 消息确认逻辑（根据具体业务实现进行处理）
        // 例如在发送消息的时候将消息标记为已发送待确认，收到回调时在此将消息标记为已确认
    }
);

// nack回调
$channel->set_nack_handler(
    function (AMQPMessage $message) {
        // 消息未成功投（根据具体业务选择重发或这不处理）
    }
);

// 声明消息队列
$channel->queue_declare($queue_name, false, true, false, false);
// 消息内容
$msg = new AMQPMessage('Hello World!');

// 发送消息到指定的消息队列中
$channel->basic_publish($msg, '', $queue_name);

echo " [Producer(Hello world)] Sent 'Hello World!'\n";

$channel->close();
$connection->close();

