<?php

require_once('../vendor/autoload.php');
require_once('../Constant.php');

use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;


class FibonacciRpcClient
{
    private $connection;
    private $channel;
    private $callback_queue;
    private $response;
    private $corr_id;
    // 回调队列名称
    private $reply_queue_name = 'rpc_reply_to';
    // rpc服务队列名称
    private $rpc_servere_queue_name = 'rpc_server_queue';

    public function __construct()
    {
        // 创建连接
        $this->connection = new AMQPStreamConnection(
            Constant::HOST,
            Constant::PORT,
            Constant::USERNAME,
            Constant::PASSWORD,
            Constant::VHOST,
            false,
            Constant::LOGIN_METHOD);
        // 建立通道
        $this->channel = $this->connection->channel();
        // 声明回调队列
        list($this->callback_queue, ,) = $this->channel->queue_declare(
            $this->reply_queue_name,
            false,
            true,
            false,
            false
        );
        // 获取回调消息
        $this->channel->basic_consume(
            $this->callback_queue,
            '',
            false,
            true,
            false,
            false,
            array(
                $this,
                'onResponse'
            )
        );
    }

    public function onResponse($rep)
    {
        // 匹配请求与响应
        if ($rep->get('correlation_id') == $this->corr_id) {
            $this->response = $rep->body;
        }
    }

    // 发起RPC请求
    public function call($n)
    {
        $this->response = null;
        $this->corr_id = uniqid();

        $msg = new AMQPMessage(
            (string)$n,
            array(
                'correlation_id' => $this->corr_id,
                'reply_to' => $this->callback_queue
            )
        );
        // 发其RPC请求
        $this->channel->basic_publish($msg, '', $this->rpc_servere_queue_name);
        while (!$this->response) {
            $this->channel->wait();
        }
        return intval($this->response);
    }
}

$fibonacci_rpc = new FibonacciRpcClient();
$response = $fibonacci_rpc->call(30);
echo ' [RPC Request get response] Got ', $response, "\n";
