<?php

require_once('../vendor/autoload.php');
require_once('../Constant.php');

use PhpAmqpLib\Connection\AMQPStreamConnection;
use PhpAmqpLib\Message\AMQPMessage;

$rpc_server_queue_name = 'rpc_server_queue';

// 创建连接
$connection = new AMQPStreamConnection(
    Constant::HOST,
    Constant::PORT,
    Constant::USERNAME,
    Constant::PASSWORD,
    Constant::VHOST,
    false,
    Constant::LOGIN_METHOD);
// 建立通道
$channel = $connection->channel();
// 声明rpc服务队列
$channel->queue_declare($rpc_server_queue_name, false, true, false, false);

// rpc服务的业务逻辑
function fib($n)
{
    if ($n == 0) {
        return 0;
    }
    if ($n == 1) {
        return 1;
    }
    return fib($n - 1) + fib($n - 2);
}

echo " [RPC Server] Awaiting RPC requests\n";
// 消费逻辑（接受PRC请求）
$callback = function ($req) {
    $n = intval($req->body);
    echo ' [RPC Server receive request] fib(', $n, ")\n";

    $msg = new AMQPMessage(
        (string)fib($n),
        array('correlation_id' => $req->get('correlation_id'))
    );
    // 向replyTo队列发送rpc请求结果（响应RPC请求）
    $req->delivery_info['channel']->basic_publish(
        $msg,
        '',
        $req->get('reply_to')
    );
    // 回复ack
    $req->ack();
};

// 设置只接收一个未确认的消息
$channel->basic_qos(null, 1, null);
// 监听RPC服务队列，等待RPC请求
$channel->basic_consume($rpc_server_queue_name, '', false, false, false, false, $callback);

while ($channel->is_open()) {
    $channel->wait();
}

$channel->close();
$connection->close();